#!/usr/bin/env python
###################
# whois - geographical map
#
# Requires matplotlib and PIL.
#
# World map 'land_shallow_topo_2048.png' from the Basemap toolkit:
#   http://matplotlib.sourceforge.net/toolkits.html
# Country coordinates from the factbook, 'geos.zip':
#  https://www.cia.gov/cia/download.html
#
# Copyright Jean-Michel Philippe (c) 2007
#   <http://philipjm.free.fr/blog/>
#
# This is open source software released under the GPL license.
# The full text of this license is found in the file 'LICENSE',
# included with this source code package.
###################

"""
whois Python module
-------------------

Geographical localisation of countries thanks to a longitude and latitude database.

Querying coordinates of a single country:

>>> whois.geomap.getLongLat('SPAIN')
(40.0, -4.0)

Drawing the world map with circles at given countries:

>>> from pylab import show
>>> whois.geomap.world({'CANADA': 48, 'FRANCE': 4, 'INDIA': 570})
>>> show()
"""

###################
# importations
import os, string, pylab, Image

###################
# constants
__ModulePath__ = os.path.dirname(__file__)
__LongLatFile__ = os.path.join(__ModulePath__, "db", "geos.txt")	# country longitude, latitude
__TranslationFile__ = os.path.join(__ModulePath__, "db", "country-translate.txt")	# whois country -> geos country

###################
# variables
CountryLocations = {}		# dictionary of (longitude, latitude) per country
CountryTranslation = {}		# translation dictionary whois country -> geos country

###################
def strCoord2Num(StrValue):
	"""
	Converts a string longitude or latitude to a float number.
	
	>>> Coord = strCoord2Num(StrValue)
	
	Input:
		- Value		= string coordinate as 'DD MM O', DD degrees, MM minutes,
					  O orientation (= 'E', 'W', 'N' or 'S')
	Output:
		- Coord		= float coordinate (degrees)
	"""
	Degree, Minutes, Direction = StrValue.strip().split(' ')
	Coord = float(Degree) + float(Minutes) / 60.0
	if Direction in ('N', 'E'):
		return Coord
	else:
		return -Coord

###################
def loadCountryLocations():
	"""
	Loads the file of country longitudes and latitudes.
	
	>>> loadCountryLocations()
	"""
	global __LongLatFile__, CountryLocations
	
	# read file
	if not os.path.isfile(__LongLatFile__):
		raise IOError, "! cannot find '%s'" % __LongLatFile__
	FileId = open(__LongLatFile__, 'r')
	Text = FileId.readlines()
	FileId.close()
	
	# get country coordinates
	CountryLocations = {}
	for Line in Text:
		if Line[0] == '#':
			continue
		Country, Latitude, Longitude = Line.strip().split(';')
		if Country.find(',') > 0:
			Country = string.join(Country.split(',')[::-1], ' ')[1:]
		CountryLocations[Country.upper()] = (strCoord2Num(Latitude), strCoord2Num(Longitude))

###################
def loadCountryTranslation():
	"""
	Loads the file of whois country to geos country translation.
	
	>>> loadCountryTranslation()
	"""
	global __TranslationFile__, CountryTranslation
	
	# read file
	CountryTranslation = {}
	if os.path.isfile(__TranslationFile__):
		FileId = open(__TranslationFile__, 'r')
		Text = FileId.readlines()
		FileId.close()
		for Line in Text:
			if Line[0] == '#':
				continue
			try:
				WhoisCountry, DBCountry = map(string.strip, Line.split('='))
				if len(DBCountry):
					CountryTranslation[WhoisCountry] = DBCountry
			except:
				pass
	else:
		print "! country translation file '%s' not found" % __TranslationFile__
		print "You should run '%s'" % os.path.join(__ModulePath__, 'db', 'country-compare.py')

###################
def getLongLat(Country):
	"""
	Returns the country longitude and latitude, 'None' if country not found.
	
	>>> Coords = getLongLat(Country)
	
	Input:
		- Country	= country name, *uppercase* only
	Output:
		- Coords	= tuple (longitude, latitude) or 'None'
	"""
	global CountryLocations, CountryTranslation
	
	if Country in CountryTranslation:
		Country = CountryTranslation[Country]
	if Country in CountryLocations:
		return CountryLocations[Country]
	else:
		return None

###################
def world(CountryStats, smin=200.0, coeff=1.0, func=None, noticks=True, c='b', marker='o', cmap=None, norm=None, vmin=None, vmax=None, alpha=1.0, linewidths=None, faceted=True, **kwargs):
	"""
	Draws the world map with circles at country location. The circles surface
	is proportional to a function of input data for each country (see below).
	
	>>> ScatterObj = world(CountryStats, smin=200.0, coeff=1.0, func=None, noticks=True, c='b', marker='o', cmap=None, norm=None, vmin=None, vmax=None, alpha=1.0, linewidths=None, faceted=True, **kwargs)
	
	Important note: saving a map as PNG significantly decreases circle surface.
	
	Input:
		- CountryStats	= dictionary of country stats, CountryStats[Country] = integer
		- smin			= minimum circle surface (pixels, world = 2048x1024)
		- coeff			= circle sizing coefficient
		- func			= input data function
		- noticks		= do not draw axis ticks
		- other args	= see matplotlib 'scatter' function documention
	Output:
		- ScatterObj	= the 'scatter' draw object
	
	Example:
	
	>>> whois.geomap.world({'CANADA': 50, 'FRANCE': 34, 'INDIA': 67})
	
	The circle surface is computed this way:
	
	S = map(func, coeff * array(CountryStats.values()) + smin)
	"""
	global __ModulePath__
	
	# compute numerical data
	Data = []
	for Country, Stats in CountryStats.items():
		Coord = getLongLat(Country)
		if Coord is None:
			pass
		else:
			Data.append( list(getLongLat(Country)) )
			Data[-1].append(Stats)
	Data = pylab.transpose(pylab.array(Data))
	
	# plot data
	World = Image.open(os.path.join(__ModulePath__, 'land_shallow_topo_2048.jpg'))
	pylab.imshow(
		World,
		origin='lower',
		extent=[-180.0, 180.0, -90.0, 90.0],
		alpha=0.75,
	)
	if c == 'data':
		c = Data[2]
	Size = coeff*Data[2]+smin
	if func is not None:
		Size = map(func, Size)
	ScatterObj = pylab.scatter(
		Data[1], Data[0], Size,
		c=c, cmap=cmap, marker=marker, norm=norm,
		vmin=vmin, vmax=vmax, alpha=alpha,
		linewidths=linewidths, faceted=faceted,
		**kwargs
	)
	pylab.axis([-180.0, 180.0, -90.0, 90.0])
	if noticks:
		pylab.xticks([])
		pylab.yticks([])
	
	return ScatterObj

###################
###################
###################

loadCountryLocations()
loadCountryTranslation()

###################
# test
if __name__ == "__main__":
	CountryNames = CountryLocations.keys()
	Index = pylab.rand(50) * len(CountryNames)
	DataSet = {}
	for k in Index.astype('i'):
		if CountryNames[k] not in DataSet:
			DataSet[CountryNames[k]] = int(1000.0 * pylab.rand(1)[0])**2
	print DataSet.keys()
	pylab.figure(None, figsize=[9, 5])
	world(DataSet, c='r', alpha=0.75, func=pylab.sqrt)
	pylab.show()
